/*********************************************
 *
 * Kinect
 *
 *  Sample wrapper for communivating with kinect
 *  (using freenect library)
 * Provides only subset of freenect functionality,
 * enough to get depth data ;)
 *
 *
 * ©2011 Z. Travnicek <travnicek@iim.cz>
 *********************************************/

package kinect;

import java.nio.ByteBuffer;
import org.openkinect.freenect.*;


public class Kinect {
    // Rozmery obrazu
    final int width = 640;
    final int height = 480;

    Context ctx = null;
    Device dev = null;

    final Object lock = new Object();

    boolean frame_changed = false;
    int [] depth_data = new int[width*height];

    /*
     * Detects and initializes kinect device (always device number 0)
     *
     * @return true if device was successfully openened, false otherwise
     */
    public boolean initializeKinect()
    {
        // Detect and open kinect
        ctx = Freenect.createContext();
        if (ctx.numDevices() > 0) {
            System.out.println("Found "+ctx.numDevices()+" kinects.");
            dev = ctx.openDevice(0);
        } else {
            System.err.println("No kinects detected.  Exiting.");
            return false;
        }
        return true;
    }

    /*
     * Closes kinect device and deletes context.
     */
    public void shutdownKinect()
    {
        if (ctx != null)
            if (dev != null) {
                dev.stopDepth();
                dev.close();
            }
        ctx.shutdown();
    }

    /*
     * Starts depth
     *
     * @return true if successfully started, false otherwise.
     */
    public boolean startKinectDepth(DepthFormat format)
    {
        if (dev == null) return false;
        // Set depth format to 11bits per pixel (with padding to 16bits)
        dev.setDepthFormat(format);
        System.out.println("Starting kinect thread");
        // Stats the depth collection
        dev.startDepth(new DepthHandler() {
            @Override
            public void onFrameReceived(DepthFormat format, ByteBuffer frame, int timestamp) {
                synchronized (lock) {
                    frame.rewind();
                    for (int i=0;i<width*height;++i) {
                        // Tady je vypocitana hloubka pro aktualni pixel (souradnice by byly x=i%width, y=i/width)
                        int val =  (frame.get() & 0xFF) + ((frame.get() & 0x07) <<8 );
                        depth_data[i] = val;
                    }
                    frame_changed = true;
                }
            }
        });
        return true;
    }

    public int[] getDepthData()
    {
        synchronized(lock) {
            frame_changed = false;
            return depth_data;
        }
    }

    public boolean isChanged()
    {
        synchronized(lock) {
            return frame_changed;
        }
    }
}
