/*********************************************
 *
 * Kinect Display
 *
 *  Sample code for visualization of depth
 *  from kinect (using freenect library)
 *
 *
 * ©2011 Z. Travnicek <travnicek@iim.cz>
 *********************************************/

package kinect;

import java.awt.*;
import java.awt.event.*;
import java.awt.Image;
import java.awt.image.MemoryImageSource;
import org.openkinect.freenect.DepthFormat;

/**
 *
 * @author worker
 */
public class Main extends Frame implements Runnable {

    Kinect device;
    // Pocet ms mezi snimky
    final int DELAY = 100;
    // Rozmery obrazu
    final int width = 640;
    final int height = 480;
    // Minimalni vzdalenost od kinectu
    final int dead_zone=300;


    Thread t  = null;
    Image frame;


    
    Boolean have_data = false;
    int [] img = new int[width*height];

    final static int [] temperature = new int[0x800];

    static {
        generate_temp();
    }

    /*
     * Constructor
     */
    public Main() {
        super( "Kinect Display" );
        device = new Kinect();
    }

    /*
     * Test if there's new frame available and
     * regenerate the image to display if needed
     */
    private void regenerateImage()
    {
        // Regenerate image if there's any change in the input data.
        if (device.isChanged()) {
            int[] depth_data = device.getDepthData();
            for (int i=0;i<width*height;++i) {
                int val = depth_data[i];
                // Clip value, so the dead zone is ignored
                int clipped_value = val>dead_zone?val-dead_zone:0;
                // Map the depth to temperature color
                img[i]=temperature[clipped_value & 0x7FF];
            }
            frame = createImage(new MemoryImageSource(width, height, img, 0, width));
            have_data = true;
        }
    }

     /*
     * Receives depth frames from kinect and prepares image for display
     */
    public void run() {
        if (!device.startKinectDepth(DepthFormat.D11BIT)) return;

        while( t!=null) {
            try {
                Thread.sleep( DELAY);
                regenerateImage();
                repaint();
            }
            catch( InterruptedException e )
            {
                System.out.println(e.toString());
            }
        }
    }

    public void start()
    {
        System.out.println("Kinect Display 1.0 ;)");

        if (!device.initializeKinect()) {
            return;
        }

        // Setting awk frame
        this.setBounds( 100, 100, width, height );
        this.setBackground( Color.RED );
        this.setVisible( true );
        this.addKeyListener(
            new KeyAdapter() {
            @Override
                public void keyReleased( KeyEvent e ) {
                    if ( e.getKeyCode()==KeyEvent.VK_ESCAPE ) { // Exit and cleanup whe ESC is pressed
                        device.shutdownKinect();
                        System.exit(0);
                    }
                }
            }
        );
        System.out.println("Starting display thread");
        // Start thread
        t = new Thread( this );
	t.start();
    }


    @Override
    public void paint( Graphics g ) {
        update(g);
    }

    /*
     * Redraw the frame
     */
    @Override
    public void update(Graphics g) {
        if (have_data) g.drawImage( frame, 0, 0, null );
    }



     /*
     * Initliaze temperature gradient map
     */
    protected static void generate_temp()
    {
        int ii, offset = 0x0;
        for (int i = 0;i<0x100;++i) {
            ii = 0xff - i;
            temperature[offset+i]=0xFFFF0000 | ii << 8 | ii;
        }
        offset+=0x100;
        for (int i = 0;i<0x100;++i) {
            //ii = 0xff - i;
            temperature[offset+i]=0xFFFF0000 | i << 8 ;
        }
        offset+=0x100;
        for (int i = 0;i<0x100;++i) {
            ii = 0xff - i;
            temperature[offset+i]=0xFF00FF00 | ii << 16 ;
        }
        offset+=0x100;
        for (int i = 0;i<0x100;++i) {
            ii = 0xff - i;
            temperature[offset+i]=0xFF00FF00 |  i;
        }
        offset+=0x100;
        for (int i = 0;i<0x100;++i) {
            ii = 0xff - i;
            temperature[offset+i]=0xFF0000FF |ii << 8;
        }
        offset+=0x100;
        for (int i = 0;i<0x100;++i) {
            ii = 0xff - i;
            temperature[offset+i]=0xFF000000 |ii ;
        }
        for (int i=offset;i<0x7ff;++i) {
            temperature[i]=0xFF000000;
        }
    }


    public static void main(String[] args) {
        Main kd = new Main();
        kd.start();
    }

}
